import hal

import gtk, sys, cairo, os
from math import pi
CONFIGPATH = os.environ['CONFIG_DIR']

#pngfile = 'vortex.me.png'
pngfile = 'sample.png'
# This is a handler file for using Gscreen's infrastructure
# to load a completely custom glade screen
# The only things that really matters is that it's saved as a GTK builder project,
# the toplevel window is caller window1 (The default name) and you connect a destroy
# window signal else you can't close down linuxcnc 
class HandlerClass:

    # This is so we can add a background pic to the window
    # this is called everytime the widget gets repainted
    def on_expose(self,widget,data=None):
        cr = widget.window.cairo_create()
        
        # Sets the operator to clear which deletes everything below 
        # where an object is drawn
        cr.set_operator(cairo.OPERATOR_CLEAR)
        # Makes the mask fill the entire window
        cr.rectangle(0.0, 0.0, *widget.get_size())
        # Deletes everything in the window (since the compositing 
        # operator is clear and mask fills the entire window
        cr.fill()
        # Set the compositing operator back to the default
        cr.set_operator(cairo.OPERATOR_OVER)
        if self.scale:
            x, y, w, h = widget.allocation
            cr.scale(1.0 *w / self.width, 1.0*h/self.height)
        cr.set_source_surface(self.img, 0, 0)
        cr.paint()

    # This will be pretty standard to gain access to everything
    # emc is for control and status of linuxcnc
    # data is important data from gscreen and linuxcnc
    # widgets is all the widgets from the glade files
    # gscreen is for access to gscreens methods
    def __init__(self, halcomp,builder,useropts,gscreen):
        self.emc = gscreen.emc
        self.data = gscreen.data
        self.widgets = gscreen.widgets
        self.gscreen = gscreen

        win = self.widgets.window1
        
        #win.set_decorated(False)
        localglade = os.path.join(CONFIGPATH,pngfile)
        print "image",localglade
        self.img = cairo.ImageSurface.create_from_png(localglade)
        self.width = self.img.get_width()
        self.height = self.img.get_height()

        # Makes the window paintable, so we can draw directly on it
        win.set_app_paintable(True)
        win.set_size_request(self.width, self.height)

        # This sets the windows colormap, so it supports transparency.
        # This will only work if the wm support alpha channel
        screen = win.get_screen()
        rgba = screen.get_rgba_colormap()
        win.set_colormap(rgba)
        
        # scaling the bitmap is possible by turning this on
        # however the fixed widget does NOT do proportional scaling
        self.scale = 1

    # erase the ready-to-home message on statusbar
    def on_hal_status_all_homed(self,widget):
        print "all-homed"
        self.data.all_homed = True
        self.widgets.statusbar1.remove_message(self.gscreen.statusbar_id,self.gscreen.homed_status_message)

    def on_metric_select_clicked(self,widget):
        pass
        #self.widgets.dro_x.set_property('display_units_mm',widget.get_active())
        #self.widgets.dro_y.set_property('display_units_mm',widget.get_active())
        #self.widgets.dro_z.set_property('display_units_mm',widget.get_active())

    def on_jog_x_pos_pressed(self,widget):
        self.do_jog(0,1)
    def on_jog_x_pos_released(self,widget):
        self.do_jog(0,0)
    def on_jog_x_neg_pressed(self,widget):
        self.do_jog(0,-1)
    def on_jog_x_neg_released(self,widget):
        self.do_jog(0,0)

    def on_jog_y_pos_pressed(self,widget):
        self.do_jog(1,1)
    def on_jog_y_pos_released(self,widget):
        self.do_jog(1,0)
    def on_jog_y_neg_pressed(self,widget):
        self.do_jog(1,-1)
    def on_jog_y_neg_released(self,widget):
        self.do_jog(1,0)

    def on_jog_z_pos_pressed(self,widget):
        self.do_jog(2,1)
    def on_jog_z_pos_released(self,widget):
        self.do_jog(2,0)
    def on_jog_z_neg_pressed(self,widget):
        self.do_jog(2,-1)
    def on_jog_z_neg_released(self,widget):
        self.do_jog(2,0)

    def do_jog(self,jointnum,cmd):
        self.emc.jogging(1)
        if self.data.current_jogincr_index == 0: # continuous jog
            self.emc.continuous_jog(jointnum,cmd)
        else:
            print "jog incremental"
            if cmd == 0: return # don't want release of button to stop jog
            jogincr = self.data.jog_increments[self.data.current_jogincr_index]
            distance = self.parse_increment(jogincr)
            self.emc.incremental_jog(jointnum,cmd,distance)

    def zero_x_event(self,widgets,event):
        print 'zero X axis'

    # This connects siganals without using glade's autoconnect method
    # in this case to destroy the window
    # it calls the method in gscreen: gscreen.on_window_destroy()
    def connect_signals(self,handlers):
        signal_list = [ ["window1","destroy", "on_window1_destroy"],
                    ]
        for i in signal_list:
            self.gscreen.widgets[i[0]].connect(i[1], self.gscreen[i[2]])
        # add button click event so we know when user has clicked the 'button'
        self.widgets.zero_x_event.set_events(gtk.gdk.BUTTON_PRESS_MASK)
        self.widgets.zero_x_event.connect("button_press_event", self.zero_x_event)

    # We don't want Gscreen to initialize it's regular widgets because this custom
    # screen doesn't have most of them. So we add this function call.
    # Since this custom screen uses gladeVCP magic for its interaction with linuxcnc
    # We don't add much to this function, but we do want to be able to change the theme so:
    # We change the GTK theme to what's in gscreen's preference file.
    # gscreen.change_theme() is a method in gscreen that changes the GTK theme of window1
    # gscreen.data.theme_name is the name of the theme from the preference file 
    # To truely be friendly, we should add a way to change the theme directly in the custom screen.
    # we also set up the statusbar and add a ready-to-home message
    def initialize_widgets(self):
        self.gscreen.init_show_windows()
        #self.gscreen.init_themes()
        #self.gscreen.change_theme(self.data.theme_name)
        self.gscreen.statusbar_id = self.widgets.statusbar1.get_context_id("Statusbar1")
        self.gscreen.homed_status_message = self.widgets.statusbar1.push(1,"Ready For Homing")

    # If we need extra HAL pins here is where we do it.
    # Note you must import hal at the top of this script to do it.
    # For gaxis there is no extra pins but since we don't want gscreen to
    # add it's default pins we added this function
    def initialize_pins(self):
        pass

    # every 50 milli seconds this gets called
    # add pass so gscreen doesn't try to update it's regular widgets.
    def periodic(self):
        pass

# standard handler call
def get_handlers(halcomp,builder,useropts,gscreen):
     return [HandlerClass(halcomp,builder,useropts,gscreen)]
